import pandas as pd

import config as cf
from read_data import read_data_f, read_scen_f
from make_folder import make_folder_f
from network import network_f
from vehicle import vehicle_f
from h_demand import n_h_demand_f
from data_generation import sample_scen_f
from find_candidate import find_candidate_f
from test_uniqueness import test_uniqueness_f
from os_test import os_test_f
from store_results import store_parameters_f, store_vehicle_parameters_f

'''
write file with instance parameters
'''

store_parameters_f()

'''
read distance and time data, spatial distribution of evacuee demand
'''
distance, time, demand_mean = read_data_f()


'''
list for storing in sample and out of sample overview data
'''

n_data_overview = {n_ins: [[] for i in {'in sample', 'out of sample'}] 
                for n_ins in cf.networks}

for n_ins in cf.networks:
    w_n_folder = make_folder_f(cf.w_folder, cf.n_names[n_ins])
    if (cf.scen_data == 'read' 
        or cf.car_spatial_dist == 'read total and distribution'):
        r_n_folder = make_folder_f(cf.r_folder, cf.n_names[n_ins])
        
    origins, G_o, p_nodes, t_final = network_f(distance, time, n_ins)
  
    for hour in cf.hours:
        w_h_folder = make_folder_f(w_n_folder, 'Hour '+str(hour+1))
        
        if (cf.scen_data == 'read' 
            or cf.car_spatial_dist == 'read total and distribution'):
            r_h_folder = make_folder_f(r_n_folder, 'Hour '+str(hour+1))
        else:
            r_h_folder = None
            
        h_demand_mean, exp_n_h_demand = n_h_demand_f(demand_mean, n_ins, hour)
        
        (s_dict, r_dict, 
         s_cars_at_node, r_cars_at_node) = vehicle_f(exp_n_h_demand, n_ins,
                                                     r_h_folder)
        
        store_vehicle_parameters_f(s_cars_at_node, r_cars_at_node, w_h_folder)
        
        if cf.scen_data == 'generate':
            scen_folder = None
            
            if cf.performOutofSampleTest:
                os_scen = [sample_scen_f(n_ins, cf.os_dist, h_demand_mean, 
                                     t_final) 
                           for sc in range(cf.noutofsample)]
    
        elif cf.scen_data == 'read':
            r_scen_folder = make_folder_f(r_h_folder,'Scenario Samples')
            scen_folder = r_scen_folder
            
            if cf.performOutofSampleTest:
                r_os_folder = make_folder_f(r_scen_folder,
                                            'Out of Sample Scenarios')
                os_scen = [read_scen_f(r_os_folder,sc) 
                           for sc in range(cf.noutofsample)]
            
        
        if cf.store_scen_data == True:
            w_scen_folder = make_folder_f(w_h_folder,'Scenario Samples')
            scen_folder = w_scen_folder

            if cf.performOutofSampleTest:    
                w_os_folder = make_folder_f(w_scen_folder,
                                            'Out of Sample Scenarios')

                for sc in range(cf.noutofsample):
                    scen_df = None
                    scen_df = pd.DataFrame(os_scen[sc])
                    scen_df.to_excel(''.join([w_os_folder,'Scenario ',
                                          str(sc+1),'.xlsx']))
        
        candidate_solns = []
        candidate_soln_data = []
        
        
        for sample in range(cf.nsample):
            if cf.findCandidateSoln:
                print('instance-', cf.n_names[n_ins], 'hour-', hour+1, 
                      'sample-', sample+1)
                (n_data_overview, 
                 s_hired_dict) = find_candidate_f(s_dict, r_dict, 
                                                  s_cars_at_node, G_o, 
                                                  p_nodes, origins, 
                                                  h_demand_mean, 
                                                  exp_n_h_demand, t_final,  
                                                  distance, n_ins, 
                                                  sample, hour, 
                                                  n_data_overview, w_n_folder, 
                                                  w_h_folder, scen_folder)
            else:
                s_hired_dict = s_dict
            
            (soln_already_found, 
             soln_data) = test_uniqueness_f(s_cars_at_node, s_hired_dict, 
                                            candidate_soln_data)
            
            if not soln_already_found:
                candidate_solns.append(s_hired_dict.keys())
                candidate_soln_data.append(soln_data)
                
                if cf.performOutofSampleTest:
                    n_data_overview = os_test_f(n_ins, hour, sample, 
                                                exp_n_h_demand,
                                                s_hired_dict, s_dict, r_dict,
                                                G_o, origins, p_nodes, os_scen,
                                                w_n_folder, w_h_folder, 
                                                n_data_overview)
                


