import config as cf

from gurobipy import *

def initialize_f(s_dict, r_dict, G_o, scenarios):
    S = s_dict.keys()
    R = r_dict.keys()
    V = [s for s in S]
    V.extend([r for r in R])
    
    cars = {}
    cars.update(s_dict)
    cars.update(r_dict)
    
    y_ind = [(v,it,jtt,sc) for v in V 
                           for it, jtt in G_o[
                                   cars[v]['origin']
                                   ].edges 
                           for sc in scenarios]
    return S, R, V, cars, y_ind

def param_f(test, name):          
    Z = Model(name)
    Z.setParam('Output_Flag',0)
    Z.setParam('time_limit',cf.timeLimit)
    
    if test == 'in':
        Z.setParam('MIPGap',cf.in_MIP_gap)
    
    elif type(test) == int:
        Z.setParam('MIPGap',cf.os_MIP_gap)
    return Z
    
def var_f(Z, 
          S, R, V, y_ind,
          scenarios, p_nodes, test):    
    b = Z.addVars(scenarios,name='b')
    d = Z.addVars(V,scenarios,name='d')
    x = Z.addVars(S,vtype=GRB.BINARY,name='x')
        
    if test == 'in' and cf.in_model_Relax:
        z = Z.addVars(R,scenarios,name='z')
        y = Z.addVars(y_ind,name='y') 
        q = Z.addVars(V,p_nodes,scenarios,name='q')
        mu = Z.addVars(scenarios,name='mu')
    
    else:
        z = Z.addVars(R,scenarios,vtype=GRB.BINARY,name='z')
        y = Z.addVars(y_ind,vtype=GRB.BINARY,name='y') 
        q = Z.addVars(V,p_nodes,scenarios,vtype=GRB.INTEGER,name='q')
        mu = Z.addVars(scenarios,vtype=GRB.INTEGER,name='mu')
    
    if test == 'in' and cf.model_type == 'cc':
        k = Z.addVars(scenarios,vtype=GRB.BINARY,name='k')
    else:
        k = None
    return b, d, x, y, z, q, mu, k