import pandas as pd
import numpy as np
import sys

def read_vehicle_dist_f(r_hour_folder, file_name, origins):
    df = pd.read_csv(r_hour_folder+file_name)
    cars_per_node = {i:0 for i in origins}
    
    for row in df.index:
        orig = df.loc[row,'Origin']
        num_cars = df.loc[row, 'Number Cars']
        cars_per_node[orig] = num_cars
    return cars_per_node

def vehicle_error_f(choice):
    print('Only one instance can be tested when we '
                  + choice + ' for vehicles.')
    sys.exit(0)

def vehicle_per_node_f(spatial_dist, origins, total_cars):
    cars_per_node = {x: int(round(spatial_dist.iat[x,0]
                                  /sum(spatial_dist.iat[y,0]
                                       for y in origins)
                                        *total_cars
                                        )
                                  ) 
                     for x in origins}
        
    unassigned_cars = int(total_cars - sum(loc_cars 
                                           for loc_cars 
                                           in cars_per_node.values())
                           )
        
    if unassigned_cars > 0:
            
        for car in range(unassigned_cars):
            cars_per_node[np.random.choice(origins)] += 1
    
    return cars_per_node

def vehicle_at_node_f(cars_per_node, origins, param, p, orig_str, car_str):
    cars_at_node = {}
    car_count = 0
    
    for orig in origins:
            cars_at_node.update({orig_str+str(orig):
                                {car_str+str(car_count+s):
                                    np.random.choice([{'cost':param[i][0],
                                                       'capacity':param[i][1],
                                                       'origin':orig_str
                                                               +str(orig)} 
                                                      for i in range(
                                                                  len(param)
                                                                  )], 
                                                     p = np.array(p).ravel())
                                    for s in range(cars_per_node[orig])
                                    }
                                    })
            car_count += cars_per_node[orig]
    
    return cars_at_node